#/usr/bin/env python2.5

"""
morrery.py
version 0.11

This is a simple PyGTK GUI applet intended to display an astronomical
orrery. The application relies heavily upon a ported version of the 
'PyEphem' library created by Brandon Craig Rhodes.

More information about PyEphem is available at:

http://rhodesmill.org/pyephem/

More information about Python and PyGTK is available at:

http://www.python.org/
http://www.pygtk.org/

Copyright (C) 2008 Darren Enns <dmenns1@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA.
"""

import sys
import gobject
import pango
import pygtk
pygtk.require('2.0')
import gtk
from gtk import gdk
import hildondesktop
import cairo
import gobject
import math
import ephem
import os
import time
from math import pi, sin, cos, asin
import ConfigParser

# set this to False to disable display of seconds and update 
# only once per minute (saves CPU cycles on the tablet)
enable_seconds = False

supports_alpha = False
sun_plot_color = [1,1,0]
moon_plot_color = [0.7,0.7,0.7]
mercury_plot_color = [0.4,0.4,0.4]
venus_plot_color = [0.6,0.6,0]
earth_plot_color = [0.2,0.2,1.0]
mars_plot_color = [1,0,0]
jupiter_plot_color = [1,0.6,0]
saturn_plot_color = [0.8,0.8,0]
uranus_plot_color = [0,0.6,0.6]
neptune_plot_color = [0,0.6,0.6]
pluto_plot_color = [0.5,0.5,0.5]
inner_orbit = 1.8
outer_orbit = 50.0

class morrery(hildondesktop.HomeItem):
	def __init__(self):
		hildondesktop.HomeItem.__init__(self)
	
		self.set_resize_type(hildondesktop.HOME_ITEM_RESIZE_BOTH)

		if text_flag == "on":
			if planets_on == "all":
				self.set_size_request(300, 440)
			else:
				self.set_size_request(300, 380)
		else:
			self.set_size_request(300, 300)

		self.connect("expose-event", self.expose)
		self.connect("screen-changed", self.screen_changed)
		self.connect ("background", self.set_timer, False)
		self.connect ("foreground", self.set_timer, True)
		self.connect ("unrealize", self.unrealize)

		# set a timeout to update the clock, depending
		# on whether we are in the foreground or background
		self.timer = None
		self.set_timer(self, True)

		self.show_all()

	def expose(self, widget, event):

		global supports_alpha
		global local
		global width, height
		global cr
		global circle_x_center, circle_y_center
		global text_height
		global text_start

		local.date = ephem.now()

		width, height = self.allocation[2], self.allocation[3]

		#Get a cairo context
		cr = widget.window.cairo_create()

		if supports_alpha == True:
			if text_flag == "on":
				cr.set_source_rgba(.0, .0, .0, 0.5) # Transparent black
			else:
				cr.set_source_rgba(1.0, 1.0, 1.0, 0.0) # Transparent white
		else:
			cr.set_source_rgb(1.0, 1.0, 1.0) # Opaque white

		# Draw the background
		cr.set_operator(cairo.OPERATOR_SOURCE)
		cr.paint()

		screen_size = self.get_size_request()
		screen_width = screen_size[0]
		screen_height = screen_size[1]
	
		font_size = 11

		text_height = 6.0
		circle_x_center = width * 0.5
		if orbit_scale == "uniform" or planets_on == "inner":
			circle_y_center = width * 0.5
		else:
			circle_y_center = width * 0.4

		text_start = height * 0.5
		if planets_on == "all":
			text_start = height * 0.7
		else:
			text_start = height * 0.8

		save = local.date
		
		size = width/30.0

		cr.set_source_rgb(1.00, 1.00, 1.00)
		cr.select_font_face("Courier", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
		
		if text_flag == "on":
			cr.move_to(width * 0.10, text_start - (text_height * 2.0))
			cr.set_source_rgb(1.00, 1.00, 1.00)
			text = "%-8s %12s %13s %8s" % ("NAME", "LONGITUDE", "LATITUDE", "DISTANCE")
			cr.text_path(text)
			cr.fill()
			cr.stroke()

			cr.move_to(width * 0.10, text_start)
			text = "%-8s %12s %13s %8s" % ("--------","------------","-------------","--------")
			cr.text_path(text)
			cr.fill()
			cr.stroke()

		cr.set_source_rgb(1.00, 1.00, 0.00)
		cr.arc(circle_x_center, circle_y_center, size, 0, 2.0*3.14)
		cr.fill()
		cr.stroke()

		cr.set_source_rgb(0.00, 0.00, 0.00)
		cr.arc(circle_x_center, circle_y_center, size, 0, 2.0*3.14)
		cr.stroke()

		cr.move_to(circle_x_center - (width*0.01), circle_y_center + (width*0.01))
		cr.text_path("S")
		cr.set_source_rgb(0.00, 0.00, 0.00)
		cr.fill()
		cr.stroke()
		
		pluto = ephem.Pluto()
		pluto.compute(local)
                pos = pluto.hlong
		distance = pluto.sun_distance
		if planets_on != "inner":
			if planets_on == "all" or planets == "outer":
				if orbit_scale == "uniform":
					radius = width * 0.45
				else:
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, pluto_plot_color, True, "P", pluto)

		neptune = ephem.Neptune()
		neptune.compute(local)
                pos = neptune.hlong
		distance = neptune.sun_distance
		if planets_on != "inner":
			if planets_on == "all" or planets == "outer":
				if orbit_scale == "uniform":
					radius = width*0.40
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, neptune_plot_color, True, "N", neptune)

		uranus = ephem.Uranus()
		uranus.compute(local)
                pos = uranus.hlong
		distance = uranus.sun_distance
		if planets_on != "inner":
			if planets_on == "all" or planets == "outer":
				if orbit_scale == "uniform":
					radius = width*0.35
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, uranus_plot_color, True, "U", uranus)

		saturn = ephem.Saturn()
		saturn.compute(local)
                pos = saturn.hlong
		distance = saturn.sun_distance
		if planets_on != "inner":
			if planets_on == "all" or planets == "outer":
				if orbit_scale == "uniform":
					radius = width*0.30
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, saturn_plot_color, True, "S", saturn)

		jupiter = ephem.Jupiter()
		jupiter.compute(local)
                pos = jupiter.hlong
		distance = jupiter.sun_distance
		if planets_on != "inner":
			if planets_on == "all" or planets == "outer":
				if orbit_scale == "uniform":
					radius = width*0.25
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, jupiter_plot_color, True, "J", jupiter)

		mars = ephem.Mars()
		mars.compute(local)
                pos = mars.hlong
		distance = mars.sun_distance
		if planets_on != "outer":
			if planets_on == "inner":
				if orbit_scale == "uniform":
					radius = width*0.40
				else:
					radius = width * 0.5 * distance / inner_orbit
			if planets_on == "all":
				if orbit_scale == "uniform":
					radius = width*0.20
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, mars_plot_color, True, "M", mars)

		sun = ephem.Sun()
		sun.compute(local)
                pos = sun.hlong
		distance = sun.earth_distance
		if planets_on != "outer":
			if planets_on == "inner":
				if orbit_scale == "uniform":
					radius = width*0.30
				else:
					radius = width * 0.5 * distance / inner_orbit
			if planets_on == "all":
				if orbit_scale == "uniform":
					radius = width*0.15
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit

			self.radial_ball(pos, radius, size, earth_plot_color, True, "E", sun)
		
		#moon = ephem.Moon()
		#moon.compute(local)

		venus = ephem.Venus()
		venus.compute(local)
                pos = venus.hlong
		distance = venus.sun_distance
		if planets_on != "outer":
			if planets_on == "inner":
				if orbit_scale == "uniform":
					radius = width*0.20
				else:
					radius = width * 0.5 * distance / inner_orbit
			if planets_on == "all":
				if orbit_scale == "uniform":
					radius = width*0.10
				else:
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, venus_plot_color, True, "V", venus)

		mercury = ephem.Mercury()
		mercury.compute(local)
                pos = mercury.hlong
		distance = mercury.sun_distance
		if planets_on != "outer":
			if planets_on == "inner":
				if orbit_scale == "uniform":
					radius = width*0.10
				else:
					radius = width * 0.5 * distance / inner_orbit
			if planets_on == "all":
				if orbit_scale == "uniform":
					radius = width*0.05
				else:
					radius = distance / outer_orbit
					radius = width * 0.5 * distance / outer_orbit
			self.radial_ball(pos, radius, size, mercury_plot_color, True, "M", mercury)

		# day section

		#timezone_offset = ephem.date(local.date)-ephem.date(ephem.localtime(local.date))
		#timezone_hour = round(-timezone_offset * 24)
		#start_of_day = ephem.date(str(local.date.tuple()[0])+'/'+str(local.date.tuple()[1])+'/'+str(local.date.tuple()[2])+' 00:00') #reset to start of day GMT
		#local.date = start_of_day+timezone_offset # compensate for differing time zone

		return False

	def radial_line(self, pos, radius, color):

		angle1a = pos * 180/pi
		angle1b = (-angle1a+180)*pi/180

		cr.set_source_rgb(color[0], color[1], color[2])

		cr.move_to(circle_x_center, circle_y_center)
		cr.rel_line_to(radius * math.cos(angle1b), radius * math.sin(angle1b))
		cr.stroke()

		return

	def radial_ball(self, pos, radius1, radius2, color, fill, text, planet):

		cr.set_dash([1],1)

		if radial_flag == "on":
			orbit_color = [1.00, 1.00, 1.00]
			self.radial_line(pos, radius1, orbit_color)

		if planet.name == "Mercury":
			planet_interval = 87.969/orbit_resolution
			planet_num = 1
		if planet.name == "Venus":
			planet_interval = 224.70069/orbit_resolution
			planet_num = 2
		if planet.name == "Sun":
			planet_interval = 365.256366/orbit_resolution
			planet_num = 3
		if planet.name == "Mars":
			planet_interval = 686.971/orbit_resolution
			planet_num = 4
		if planet.name == "Jupiter":
			planet_interval = 4331.572/orbit_resolution
			planet_num = 5
		if planet.name == "Saturn":
			planet_interval = 10832.327/orbit_resolution
			planet_num = 6
		if planet.name == "Uranus":
			planet_interval = 30799.095/orbit_resolution
			planet_num = 7
		if planet.name == "Neptune":
			planet_interval = 60190/orbit_resolution
			planet_num = 8
		if planet.name == "Pluto":
			planet_interval = 90613.3055/orbit_resolution
			planet_num = 9

		if planets_on == "outer":
			planet_num = planet_num - 4

		if text_flag == "on":
			cr.move_to(width * 0.10, text_start + (text_height * 2.0 * planet_num))
			cr.set_source_rgb(1.00, 1.00, 1.00)

			if planet.name != "Sun":
				planet_distance = planet.sun_distance
				planet_name = planet.name
			else:
				planet_distance = planet.earth_distance
				planet_name = "Earth"

			cr.text_path("%-8s %12s %13s %8s" % (planet_name, planet.hlong, planet.hlat, round(planet_distance,3)))
			cr.fill()
			cr.stroke()

			cr.set_source_rgb(color[0], color[1], color[2])
			cr.arc(width * 0.05, text_start + (text_height * 2.0 * planet_num), radius2 * 0.6, 0, 2.0*3.14)
			cr.fill()	
			cr.stroke()

			cr.set_source_rgb(0.00, 0.00, 0.00)
			cr.arc(width * 0.05, text_start + (text_height * 2.0 * planet_num), radius2 * 0.6, 0, 2.0*3.14)
			cr.stroke()

			cr.move_to(width * 0.05 - (width*0.01), text_start + (text_height * 2.0 * planet_num + (width*0.01)))
			cr.text_path(text)
			cr.set_source_rgb(1.00, 1.00, 1.00)
			cr.fill()
			cr.stroke()

		if orbit_flag == "on":
			if orbit_scale == "uniform":
				cr.set_source_rgb(1.00, 1.00, 1.00)
				cr.arc(circle_x_center, circle_y_center, radius1, 0, 2.0*3.14)
				cr.stroke()
			else:
				for degrees in range(0,orbit_resolution,1):
					planet.compute(local)
					orbit_pos = planet.hlong
					if planet.name != "Sun":
						planet_distance = planet.sun_distance
					else:
						planet_distance = planet.earth_distance
					if planets_on == "all" or planets_on == "outer":
						radius = width * 0.5 * planet_distance / outer_orbit
					if planets_on == "inner":
						radius = width * 0.5 * planet_distance / inner_orbit
					angle1a = orbit_pos * 180/pi
					angle1b = (-angle1a+180)*pi/180
					cr.set_source_rgb(1.00, 1.00, 1.00)
					cr.arc(circle_x_center + (radius * math.cos(angle1b)), circle_y_center + (radius * math.sin(angle1b)), 1, 0, 2.0*3.14)
					cr.fill()
					cr.stroke()
					local.date = local.date + planet_interval

			local.date = ephem.now()

		angle1a = pos * 180/pi
		angle1b = (-angle1a+180)*pi/180

		cr.set_source_rgb(color[0], color[1], color[2])
		cr.arc(circle_x_center + (radius1 * math.cos(angle1b)), circle_y_center + (radius1 * math.sin(angle1b)), radius2, 0, 2.0*3.14)
		if fill == True:
			cr.fill()	
		cr.stroke()

		cr.set_source_rgb(0.00, 0.00, 0.00)
		cr.arc(circle_x_center + (radius1 * math.cos(angle1b)), circle_y_center + (radius1 * math.sin(angle1b)), radius2, 0, 2.0*3.14)
		cr.stroke()

		cr.move_to(circle_x_center + (radius1 * math.cos(angle1b)) - (width*0.01), circle_y_center + (radius1 * math.sin(angle1b)) + (width*0.01))
		cr.text_path(text)
		cr.set_source_rgb(1.00, 1.00, 1.00)
		cr.fill()
		cr.stroke()

		return

	def screen_changed(self, widget, old_screen=None):
		global supports_alpha
		# print "screen changed"

		# To check if the display supports alpha channels, get the colormap
		screen = self.get_screen()
		colormap = screen.get_rgba_colormap()
		if colormap == None:
			# print 'Your screen does not support alpha channels!'
			colormap = screen.get_rgb_colormap()
			supports_alpha = False
		else:
			# print 'Your screen supports alpha channels!'
			supports_alpha = True
    
		# Now we have a colormap appropriate for the screen, use it
		self.set_colormap(colormap)
    
		return False
	
	def unrealize(self, widget, date=None):
		# cancel timeout
		if self.timer:
			v = gobject.source_remove(self.timer)
			print "canceled pyclock timeout:", v
			self.timer = None

	def set_timer(self, widget, on):

		# when called first time from __init__ widget is None
		if self.timer != None:
			# print "removing old timer"
			gobject.source_remove(self.timer)
		#else:
		if on:
			# print "creating new timer"
			#delay = 1000 if enable_seconds else 60000
			delay = 86400000
			self.timer = gobject.timeout_add(delay, self.update)
			# repaint immediately when coming to the foreground
			self.update()

	def update(self):
		# print "updating morrery"
		self.queue_draw()
		return True

def hd_plugin_get_objects():
	global local
	global text_flag
	global orbit_flag
	global planets_on
	global orbit_scale
	global orbit_resolution
	global radial_flag

	config = ConfigParser.ConfigParser()
	ini_file = '/usr/lib/hildon-desktop/morrery.ini'
	config.read(ini_file)
	text_flag = config.get("style","text")
	orbit_flag = config.get("style","orbit")
	radial_flag = config.get("style","radial")
	planets_on = config.get("style","planets")
	orbit_scale = config.get("style","scale")
	orbit_resolution = float(config.get("style","resolution"))

	local = ephem.Observer()

	plugin = morrery()
	return [plugin]

